#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <linux/types.h>
#include <linux/kd.h>
#include <linux/keyboard.h>

#define VERSION "0.81"

static int fd;

int get_bind(unsigned char index, unsigned char table) {
	struct kbentry ke;
	ke.kb_index = index;
	ke.kb_table = table;
	if (ioctl(fd, KDGKBENT, (unsigned long)&ke)) {
		fprintf(stderr, "KDGKBENT at index %d in table %d", index, table);
		perror("");
		exit(1);
	}
	return ke.kb_value;
}

void usage(void) {
	fprintf(stderr, "mktable version " VERSION "\n");
	fprintf(stderr, "usage: mktable\n");
	exit(1);
}

main (int argc, char *argv[]) {
	int i;
	int j;

	struct kbsentry fbuf;
	int lentab[NR_FUNC];
	u_char *p;

	if (argc != 1)
		usage();

	if ((fd = open("/dev/console", O_RDONLY)) < 0) {
		perror("when opening /dev/console");
		exit(1);
	}

	printf("/* Automatically generated by mktable */\n");
	printf("/* Do not edit this file!             */\n");
	printf("\n#include <linux/types.h>");
	printf("\n#include <linux/keyboard.h>\n\n");
	printf("u_short key_map[NR_KEYMAPS][NR_KEYS] = {\n\t");
	for (j = 0; j < NR_KEYMAPS; j++) {
		printf("{");
		for (i = 0; i < NR_KEYS; i++) {
			if (!(i % 8))
				printf("\n\t");
			printf("0x%04x,\t", get_bind(i, j));
		}
		printf("\n\t}, ");
	}
	printf("\n};\n\n");

	printf("char func_buf[FUNC_BUFSIZE] = {\n");
	for (i = 0; i < NR_FUNC; i++) {
		fbuf.kb_func = i;
		if (ioctl(fd, KDGKBSENT, (unsigned long)&fbuf)) {
			fprintf(stderr, "KDGKBSENT at index %d", i);
			perror("");
			exit(1);
		}
		printf("\t");
		for (p = fbuf.kb_string; *p; p++)
			if (isgraph(*p))
				printf("'%c', ", *p);
			else
				printf("'\\%03o', ", *p);
		printf("0, \n");
		lentab[i] = p - fbuf.kb_string + 1;
	}
	printf("};\n\n");

	printf("char *func_table[NR_FUNC] = {\n");
	j = 0;
	for (i = 0; i < NR_FUNC; i++) {
		printf("\tfunc_buf + %d,\n", j);
		j += lentab[i];
	}
	printf("};\n");

	exit(0);
}
